<?php

/**
 * @file
 * Contains the Collection class.
 */

namespace Drupal\collection_class;

/**
 * Class Collection.
 */
class Collection implements \Countable, \IteratorAggregate {

  /**
   * Items to collect.
   *
   * @var array
   */
  private $items;

  /**
   * Collection constructor.
   *
   * @param mixed $items
   *   Items to collect.
   */
  public function __construct($items = array()) {
    $this->items = is_array($items) ? $items : $this->getArrayableItems($items);
  }

  /**
   * {@inheritdoc}
   */
  public function __toString() {
    return $this->toJson();
  }

  /**
   * Return all items.
   *
   * @return array
   *   All items.
   */
  public function all() {
    return $this->items;
  }

  /**
   * Return the array keys.
   *
   * @return array|Collection
   *   An array or Collection of keys.
   */
  public function keys() {
    return new static(array_keys($this->items));
  }

  /**
   * Determines if the collection is empty.
   *
   * @return bool
   *   TRUE or FALSE.
   */
  public function isEmpty() {
    return empty($this->items);
  }

  /**
   * Count the number of items.
   *
   * @return int
   *   The number of items.
   */
  public function count() {
    return count($this->items);
  }

  /**
   * Return the first item.
   *
   * @return mixed
   *   The first item.
   */
  public function first() {
    return array_shift($this->items);
  }

  /**
   * Return the last item.
   *
   * @return mixed
   *   The last item.
   */
  public function last() {
    return array_pop($this->items);
  }

  /**
   * Run a callback on each item.
   *
   * @param callable $callback
   *   A callable function to run.
   *
   * @return $this
   */
  public function each(callable $callback) {
    foreach ($this->items as $key => $item) {
      $callback($item, $key);
    }

    return $this;
  }

  /**
   * Filter the items within the collection.
   *
   * @param callable|NULL $callback
   *   A callable function to run.
   *
   * @return $this
   */
  public function filter(callable $callback = NULL) {
    if ($callback) {
      return new static(array_filter($this->items, $callback));
    }

    return new static(array_filter($this->items));
  }

  /**
   * Applies the callback to the elements of the given arrays.
   *
   * @param callable $callback
   *   A callable function.
   *
   * @return $this
   *
   * @see array_map()
   */
  public function map(callable $callback) {
    $keys = $this->keys()->all();
    $items = array_map($callback, $this->items, $keys);

    return new static(array_combine($keys, $items));
  }

  /**
   * Merge additional items into the collection.
   *
   * @param mixed $items
   *   Items to merge.
   *
   * @return $this
   */
  public function merge($items) {
    return new static(array_merge($this->items, $this->getArrayableItems($items)));
  }

  /**
   * Return the collection as JSON.
   *
   * @return string
   *   The collection.
   */
  public function toJson() {
    return json_encode($this->items);
  }

  /**
   * {@inheritdoc}
   */
  public function getIterator() {
    return new \ArrayIterator($this->items);
  }

  /**
   * Find arrayable items.
   *
   * @param mixed $items
   *   An array or collection of items.
   *
   * @return array
   *   An array of items.
   */
  private function getArrayableItems($items) {
    if ($items instanceof Collection) {
      return $items->all();
    }

    return $items;
  }

}
